import json
import time
from typing import Any, Dict, List

import attrs
import requests
from gql import gql
from jsonschema import ValidationError, validate
from vals.sdk.auth import _get_auth_token
from vals.sdk.exceptions import ValsException
from vals.sdk.util import RUN_SCHEMA_PATH, be_host, fe_host, get_client

# Rudimentary cache for pulling params
_default_params = None


run_param_info_query = gql(
    """
query RunParamInfo {
  runParameterInfo
}
"""
)


def _get_default_parameters() -> Dict[str, Any]:
    global _default_params
    if _default_params is not None:
        return _default_params

    response = get_client().execute(run_param_info_query)
    param_info = response["runParameterInfo"]

    _default_params = {}
    for k, v in param_info.items():
        default_val = v["default"]
        if v["type"] == "select":
            default_val = default_val["value"]
        _default_params[k] = default_val

    return _default_params


start_run_mutation = gql(
    """
    mutation StartRunMutation(
        $test_suite_id: String!
        $parameters: GenericScalar!
        $qa_set_id: String = null
    ) {
        startRun(
            testSuiteId: $test_suite_id, 
            parameters: $parameters,
            qaSetId: $qa_set_id
        ) {
            runId
        }
    }
    """
)


def start_run(
    suite_id: str,
    parameters: Dict[Any, Any] = {},
    qa_set_id: str | None = None,
) -> str:
    """
    Method to start a run.

    suite_id: The ID of the suite to run.
    parameters: Any run parameters. Examples include 'use_golden_output'
    'threads', etc.

    Returns the run id.
    """
    try:
        with open(RUN_SCHEMA_PATH, "r") as f:
            schema = json.load(f)
            default_params = _get_default_parameters()
            parameters = {**default_params, **parameters}

            # Description isn't included by default
            if "description" not in parameters:
                parameters["description"] = "Ran with PRL SDK."

            # Validation
            validate(instance=parameters, schema=schema)

    except ValidationError as e:
        raise ValsException(
            f"Config file provided did not conform to JSON schema. Message: {e.message}"
        )

    response = get_client().execute(
        start_run_mutation,
        variable_values={
            "test_suite_id": suite_id,
            "parameters": parameters,
            "qa_set_id": qa_set_id,
        },
    )
    run_id = response["startRun"]["runId"]
    return run_id


def get_csv(run_id: str) -> bytes:
    """
    A method to pull a CSV from a run result.

    Returns: The CSV, as bytes.
    """
    response = requests.post(
        url=f"{be_host()}/export_results_to_file/?run_id={run_id}",
        headers={"Authorization": _get_auth_token()},
    )

    if response.status_code != 200:
        raise ValsException("Received Error from Vals Servers: " + response.text)

    return response.content


def get_run_results(
    include_archived: bool = False, suite_id: str = None
) -> List[Dict[str, Any]]:
    query = gql(
        f"""
        query MyQuery {{
          runs(archived: {"null" if include_archived else "false"}, suiteId: {"null" if suite_id is None else '"' + suite_id + '"'}) {{
            runId
            passPercentage
            status
            runId
            textSummary
            timestamp
            archived
            parameters
            testSuite {{
              title
            }}
          }}
        }}
    """
    )
    results = get_client().execute(query)["runs"]

    for result in results:
        result["parameters"] = json.loads(result["parameters"])
    # TODO: Error Handling

    return results


def run_status(run_id: str) -> str:
    """
    For a given run, returns the run status, either 'in_progress', 'success', or 'error'.
    """
    query = gql(
        f"""
        query MyQuery {{
            run(runId: "{run_id.strip()}") {{
                status
            }}
        }}
        """
    )
    results = get_client().execute(query)
    status = results["run"]["status"]
    if status == "":
        status = "in_progress"

    return status


def run_summary(run_id: str) -> Dict[str, Any]:
    """
    Produces the top-level analytics and summary data for a given run (text summary, pass percentage, etc.)
    """
    query = gql(
        f"""          
        query MyQuery {{
            run(runId: "{run_id.strip()}") {{
                status
                archived
                parameters
                textSummary
                timestamp
                completedAt
                archived
                passPercentage
                passPercentageWithOptional
                humanEvalAccuracy
                humanEvalF1
                humanEvalPhi
                humanEvalMean
                humanEvalCoverage
                checkResultSummary
            }}
        }}
        """
    )
    results = get_client().execute(query)

    # TODO: Make a non-json, pretty version if useful
    dict_result = results["run"]
    dict_result["parameters"] = json.loads(dict_result["parameters"])
    dict_result["checkResultSummary"] = json.loads(dict_result["checkResultSummary"])

    return dict_result


def _pull_test_results(run_id: str) -> Dict[str, Any]:
    query = gql(
        f"""
        query MyQuery2 {{
                testResults(runId: "{run_id}") {{
                  id
                  llmOutput
                  passPercentage
                  passPercentageWithOptional
                  resultJson
                  humanEval
                  humanFeedback
                  test {{
                    testId
                    inputUnderTest
                  }}
                  metadata
                }}
              }}
    """
    )
    results = get_client().execute(query)["testResults"]
    for result in results:
        result["resultJson"] = json.loads(result["resultJson"])

    return results


def pull_run_results_json(run_id: str) -> Dict[str, Any]:
    """
    Pull all data about a run as JSON, including both the summary
    and the test results
    """
    summary = run_summary(run_id)
    test_results = _pull_test_results(run_id)
    return {**summary, "results": test_results}


def wait_for_run_completion(run_id: str) -> str:
    """
    Block a process until a given run has finished running.

    """
    time.sleep(5)
    status = "in_progress"
    while status == "in_progress":
        status = run_status(run_id)
        time.sleep(1)

    return status


def get_run_url(run_id: str) -> str:
    """
    Utility function to transform a run id to a viewable Vals AI URL.
    """
    return f"{fe_host()}/results?run_id={run_id}"


@attrs.define
class Metadata:
    """
    Metadata for a question answer pair.
    """

    in_tokens: int
    out_tokens: int
    duration_seconds: float

    def to_graphql_dict(self) -> Dict[str, int | float]:
        # needs to be in camelcase for graphql
        return {
            "inTokens": self.in_tokens,
            "outTokens": self.out_tokens,
            "durationSeconds": self.duration_seconds,
        }


@attrs.define
class QuestionAnswerPair:
    """
    TODO: We should read this type directly from the backend.
    """

    input_under_test: str
    file_ids: List[str]
    context: Dict[str, Any]
    llm_output: str
    metadata: Metadata
    test_id: str


def _create_question_answer_set(
    test_suite_id: str,
    pairs: List[QuestionAnswerPair],
    parameters: Dict[str, Any] = {},
    model_id: str = "",
):
    # Define the mutation
    mutation = gql(
        """
    mutation CreateQuestionAnswerSet($testSuiteId: String!, $questionAnswerPairs: [QuestionAnswerPairInputType!]!, $parameters: GenericScalar!, $modelId: String!) {
        createQuestionAnswerSet(
            testSuiteId: $testSuiteId,
            questionAnswerPairs: $questionAnswerPairs,
            parameters: $parameters,
            modelId: $modelId
        ) {
            questionAnswerSet {
                id
            }
        }
    }
    """
    )

    # Get the GraphQL client
    client = get_client()

    # Prepare the variables
    variables = {
        "testSuiteId": test_suite_id,
        "questionAnswerPairs": [
            {
                "inputUnderTest": pair.input_under_test,
                "fileIds": pair.file_ids,
                "context": pair.context,
                "llmOutput": pair.llm_output,
                "metadata": pair.metadata.to_graphql_dict(),
                "testId": pair.test_id,
            }
            for pair in pairs
        ],
        "parameters": parameters,
        "modelId": model_id,
    }

    # Execute the mutation
    result = client.execute(mutation, variable_values=variables)

    # Return the created QuestionAnswerSet ID
    return result["createQuestionAnswerSet"]["questionAnswerSet"]["id"]



import os

from gql import Client, gql
from gql.transport.requests import RequestsHTTPTransport

from .auth import _get_auth_token, _get_region

VALS_ENV = os.getenv("VALS_ENV")
SCHEMA_PATH = os.path.join(os.path.dirname(__file__), "jsonschemas")
SUITE_SCHEMA_PATH = os.path.join(SCHEMA_PATH, "suiteschema.json")
RUN_SCHEMA_PATH = os.path.join(SCHEMA_PATH, "run_params_schema.json")


def be_host():
    region = _get_region()
    if region == "eu-north-1":
        return "https://europebe.playgroundrl.com"
    if VALS_ENV == "LOCAL":
        return "http://localhost:8000"
    if VALS_ENV == "DEV":
        return "https://devbe.playgroundrl.com"

    return "https://prodbe.playgroundrl.com"


def fe_host():
    region = _get_region()
    if region == "eu-north-1":
        return "https://eu.platform.vals.ai"
    if VALS_ENV == "LOCAL":
        return "http://localhost:3000"
    if VALS_ENV == "DEV":
        return "https://dev.platform.vals.ai"

    return "https://platform.vals.ai"


def get_client():
    # We do not share clients because it kills multithreading, instead we just
    # create a new client each request.

    transport = RequestsHTTPTransport(
        url=f"{be_host()}/graphql/",
        headers={"Authorization": _get_auth_token()},
        verify=VALS_ENV != "LOCAL",
    )

    client_ = Client(transport=transport, fetch_schema_from_transport=True)
    return client_


def list_rag_suites():
    query = gql(
        f"""
        query getRagSuites {{
            ragSuites {{
            id
            org
            path
            query
            }}
            }}
        """
    )
    response = get_client().execute(query)

    # TODO: Error check
    return response["ragSuites"]



import asyncio
import json
from io import BytesIO
from typing import Any

from vals.sdk.sdk import read_docx
from vals.sdk.v2.suite import Check, Suite, Test


async def run_with_model_under_test():
    """Run the suite on a stock model, gpt-4o-mini"""
    suite = Suite(
        title="Test Suite",
        global_checks=[Check(operator="grammar")],
        tests=[
            Test(
                input_under_test="What is QSBS?",
                checks=[Check(operator="equals", criteria="QSBS")],
            ),
        ],
    )
    await suite.create()
    run = await suite.run(model_under_test="gpt-4o-mini", wait_for_completion=True)

    print(f"Run URL: {run.url}")
    print(f"Pass percentage: {run.pass_percentage}")

    # Can save the results to a CSV file.
    run.to_csv("out.csv")


async def run_with_function():
    """Run the suite on a custom model function."""
    suite = Suite(
        title="Test Suite",
        tests=[
            Test(
                input_under_test="What is QSBS?",
                checks=[Check(operator="equals", criteria="QSBS")],
            ),
        ],
    )
    await suite.create()

    def function(input_under_test: str) -> str:
        # This would be replaced with your custom model.
        return input_under_test + "!!!"

    run = await suite.run(model_function=function)

    print(f"Run URL: {run.url}")
    print(f"Pass percentage: {run.pass_percentage}")


async def run_with_function_context_and_files():
    """Run the suite with context and files."""
    suite = Suite(
        title="Test Suite",
        tests=[
            Test(
                input_under_test="What is QSBS?",
                checks=[Check(operator="equals", criteria="QSBS")],
                context={
                    "message_history": [
                        {"role": "user", "content": "What is QSBS?"},
                        {"role": "assistant", "content": "QSBS is a company."},
                    ]
                },
                files_under_test=["data_files/postmoney_safe.docx"],
            ),
        ],
    )
    await suite.create()

    def function(
        input_under_test: str, files: dict[str, BytesIO], context: dict[str, Any]
    ) -> str:
        # Your LLM would leverage the context, the files, and the input_under_test
        # to return a response.
        return input_under_test

    run = await suite.run(model_function=function, wait_for_completion=True)

    print(f"Run URL: {run.url}")
    print(f"Pass percentage: {run.pass_percentage}")


async def all():
    await run_with_model_under_test()
    await run_with_function()
    await run_with_function_context_and_files()


if __name__ == "__main__":
    asyncio.run(all())



"""
Contains types we define explicitly, as opposed to those that
are auto-generated based on the GraphQL schema. 

These are meant to be user-facing. 
"""

import datetime
import json
from enum import Enum
from io import BytesIO
from typing import Any, Callable, Literal

from pydantic import BaseModel
from vals.graphql_client.get_test_data import GetTestDataTests
from vals.graphql_client.get_test_suites_with_count import (
    GetTestSuitesWithCountTestSuitesWithCountTestSuites,
)
from vals.graphql_client.input_types import TestMutationInfo
from vals.graphql_client.pull_run import PullRunTestResults

SimpleModelFunctionType = Callable[[str], str]

ModelFunctionWithFilesAndContextType = Callable[
    [str, dict[str, BytesIO], dict[str, Any]], str
]

ModelFunctionType = SimpleModelFunctionType | ModelFunctionWithFilesAndContextType


class TestSuiteMetadata(BaseModel):
    """
    Data returned about a test suite when we are calling the
    list_test_suites() function - does not include tests, global
    checks, etc.
    """

    id: str
    title: str
    description: str
    created: datetime.datetime
    creator: str
    last_modified_by: str
    last_modified_at: datetime.datetime
    folder_id: str | None
    folder_name: str | None

    # TODO: Often, in this file, we're mapping back and forth between our custom types
    # and the auto-generated types. There is probably a better solution for this.
    @classmethod
    def from_graphql(
        cls, graphql_suite: GetTestSuitesWithCountTestSuitesWithCountTestSuites
    ) -> "TestSuiteMetadata":
        return cls(
            id=graphql_suite.id,
            title=graphql_suite.title,
            description=graphql_suite.description,
            created=graphql_suite.created,
            creator=graphql_suite.creator,
            last_modified_by=graphql_suite.last_modified_by,
            last_modified_at=graphql_suite.last_modified_at,
            folder_id=graphql_suite.folder.id if graphql_suite.folder else None,
            folder_name=graphql_suite.folder.name if graphql_suite.folder else None,
        )


class Example(BaseModel):
    """
    In-context example modifier
    """

    type: Literal["positive", "negative"]
    text: str


class ConditionalCheck(BaseModel):
    """
    Predicate / conditional check modifier.
    """

    operator: str
    criteria: str


class CheckModifiers(BaseModel):
    optional: bool | None = None
    """Do not include this check towards the final pass percentage"""

    severity: float | None = None
    """Relative weight of this check - see documentation."""

    examples: list[Example] | None = None
    """In-context examples for the check"""

    extractor: str | None = None
    """Extract certain aspects of the output before the check is evaluated"""

    conditional: ConditionalCheck | None = None
    """Only run this check if another check evaluates to true."""

    @classmethod
    def from_graphql(cls, modifiers_dict: dict) -> "CheckModifiers":
        """Internal method to translate from what we receive from GraphQL to the CheckModifiers class."""
        if not modifiers_dict:
            return cls()

        examples = None
        if modifiers_dict.get("examples", None):
            examples = [Example(**example) for example in modifiers_dict["examples"]]

        conditional = None
        if modifiers_dict.get("conditional"):
            conditional = ConditionalCheck(**modifiers_dict["conditional"])

        return cls(
            optional=modifiers_dict.get("optional", False),
            severity=modifiers_dict.get("severity"),
            examples=examples,
            extractor=modifiers_dict.get("extractor"),
            conditional=conditional,
        )


class Check(BaseModel):
    operator: str
    """Operator - see documentation for the full list."""

    criteria: str = ""
    """Criteria for the check - only required if it is not a unary operator."""

    modifiers: CheckModifiers = CheckModifiers()
    """Optional additional modifiers for the check."""

    @classmethod
    def from_graphql(cls, check_dict: dict) -> "Check":
        """Internal method to translate from what we receive from GraphQL to the Check class displayed to the user."""
        modifiers = CheckModifiers.from_graphql(check_dict.get("modifiers", {}))

        return cls(
            operator=check_dict["operator"],
            criteria=check_dict["criteria"],
            modifiers=modifiers,
        )


class Test(BaseModel):
    id: str = "0"
    """Internal id of the test"""

    cross_version_id: str = ""
    """Internal id that stays constant across versions."""

    input_under_test: str
    """Input to the LLM"""

    checks: list[Check]
    """List of checks to apply to the LLM's output"""

    golden_output: str = ""
    """ Expected output of the LLM - can be used instead of or in-conjuction with checks"""

    tags: list[str] = []
    """Tags for the test"""

    context: dict[str, Any] = {}
    """Arbitrary additional context to be used as input for the test."""

    files_under_test: list[str] = []
    """Local file paths to upload as part of the test input - i.e. documents, etc."""

    _file_ids: list[str] = []
    """This is the *internal* representation of a file, as stored on the server. You generally should not edit or use these. """

    @classmethod
    def from_graphql_test(cls, graphql_test: GetTestDataTests) -> "Test":
        """Internal method to translate from what we receive from GraphQL to the Test class displayed to the user."""
        test = cls(
            id=graphql_test.test_id,
            input_under_test=graphql_test.input_under_test,
            cross_version_id=graphql_test.cross_version_id,
            tags=json.loads(graphql_test.tags),
            context=json.loads(graphql_test.context),
            golden_output=graphql_test.golden_output,
            checks=[
                Check.from_graphql(check) for check in json.loads(graphql_test.checks)
            ],
        )
        test._file_ids = json.loads(graphql_test.file_ids)
        return test

    def to_test_mutation_info(self, test_suite_id: str) -> TestMutationInfo:
        """Internal method to translate from the Test class to the TestMutationInfo class."""
        return TestMutationInfo(
            test_suite_id=test_suite_id,
            test_id=self.id,
            input_under_test=self.input_under_test,
            checks=json.dumps(
                [check.model_dump(exclude_none=True) for check in self.checks]
            ),
            tags=self.tags,
            context=json.dumps(self.context),
            golden_output=self.golden_output,
            file_ids=self._file_ids,
        )


class Metadata(BaseModel):
    in_tokens: int
    out_tokens: int
    duration_seconds: float


class CheckResult(BaseModel):
    """Result of evaluation for a single check."""

    # Same as the input fields.
    operator: str
    criteria: str
    severity: float
    modifiers: CheckModifiers
    is_global: bool

    auto_eval: int
    """Binary pass / fail of the check, 0 for fail, 1 for pass"""

    feedback: str
    """Autogenerated free-text feedback for the check"""


class TestResult(BaseModel):
    """Result of evaluation for a single test."""

    id: str
    input_under_test: str

    llm_output: str
    """Output produced by the LLM"""

    metadata: Metadata | None

    pass_percentage: float
    """Percent of passing checks for the test"""

    check_results: list[CheckResult]
    """Results for every check"""

    @classmethod
    def from_graphql(cls, graphql_test_result: PullRunTestResults) -> "TestResult":
        return cls(
            id=graphql_test_result.id,
            input_under_test=graphql_test_result.test.input_under_test,
            llm_output=graphql_test_result.llm_output,
            pass_percentage=graphql_test_result.pass_percentage,
            check_results=[
                CheckResult(**check_result)
                for check_result in json.loads(graphql_test_result.result_json)
            ],
            metadata=(
                Metadata(**json.loads(graphql_test_result.metadata))
                if graphql_test_result.metadata
                else None
            ),
        )


class RunStatus(str, Enum):
    """Status of a run: 'in_progress', 'completed', or 'success'."""

    IN_PROGRESS = "in_progress"
    COMPLETED = "completed"
    SUCCESS = "success"



import os

from openai import OpenAI
from vals.sdk.sdk import patch, run_evaluations

client = patch(OpenAI(api_key=os.environ.get("OPEN_AI_KEY")))

# Before running this, create a test suite on the Vals AI platform


def test_function(test_input: str):
    prompt = "You are a pirate, answer in the speaking style of a pirate.\n\n"
    temp = 0.2

    gpt_client = OpenAI(api_key=os.environ.get("OPEN_AI_KEY"))
    response = gpt_client.chat.completions.create(
        model="gpt-4",
        messages=[{"role": "user", "content": prompt + test_input}],
        temperature=temp,
    )
    return response.choices[0].message.content


run_id = run_evaluations(
    # Replace with the link suite link
    "https://platform.vals.ai/view?test_suite_id=xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx",
    test_function,
)



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class CreateTestSuite(BaseModel):
    update_test_suite: Optional["CreateTestSuiteUpdateTestSuite"] = Field(
        alias="updateTestSuite"
    )


class CreateTestSuiteUpdateTestSuite(BaseModel):
    test_suite: Optional["CreateTestSuiteUpdateTestSuiteTestSuite"] = Field(
        alias="testSuite"
    )


class CreateTestSuiteUpdateTestSuiteTestSuite(BaseModel):
    description: str
    id: str
    org: str
    title: str


CreateTestSuite.model_rebuild()
CreateTestSuiteUpdateTestSuite.model_rebuild()



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, Dict, List, Optional, Union

from .add_batch_tests import AddBatchTests
from .async_base_client import AsyncBaseClient
from .base_model import UNSET, UnsetType
from .create_or_update_test_suite import CreateOrUpdateTestSuite
from .create_question_answer_set import CreateQuestionAnswerSet
from .delete_test_suite import DeleteTestSuite
from .get_operators import GetOperators
from .get_test_data import GetTestData
from .get_test_suite_data import GetTestSuiteData
from .get_test_suites_with_count import GetTestSuitesWithCount
from .input_types import QuestionAnswerPairInputType, TestMutationInfo
from .list_runs import ListRuns
from .pull_run import PullRun
from .remove_old_tests import RemoveOldTests
from .run_param_info import RunParamInfo
from .run_status import RunStatus
from .start_run import StartRun
from .update_global_checks import UpdateGlobalChecks


def gql(q: str) -> str:
    return q


class Client(AsyncBaseClient):
    async def create_question_answer_set(
        self,
        test_suite_id: str,
        question_answer_pairs: List[QuestionAnswerPairInputType],
        parameters: Any,
        model_id: str,
        **kwargs: Any
    ) -> CreateQuestionAnswerSet:
        query = gql(
            """
            mutation CreateQuestionAnswerSet($testSuiteId: String!, $questionAnswerPairs: [QuestionAnswerPairInputType!]!, $parameters: GenericScalar!, $modelId: String!) {
              createQuestionAnswerSet(
                testSuiteId: $testSuiteId
                questionAnswerPairs: $questionAnswerPairs
                parameters: $parameters
                modelId: $modelId
              ) {
                questionAnswerSet {
                  id
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "testSuiteId": test_suite_id,
            "questionAnswerPairs": question_answer_pairs,
            "parameters": parameters,
            "modelId": model_id,
        }
        response = await self.execute(
            query=query,
            operation_name="CreateQuestionAnswerSet",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return CreateQuestionAnswerSet.model_validate(data)

    async def start_run(
        self,
        test_suite_id: str,
        parameters: Any,
        qa_set_id: Union[Optional[str], UnsetType] = UNSET,
        run_name: Union[Optional[str], UnsetType] = UNSET,
        **kwargs: Any
    ) -> StartRun:
        query = gql(
            """
            mutation startRun($test_suite_id: String!, $parameters: GenericScalar!, $qa_set_id: String = null, $run_name: String = null) {
              startRun(
                testSuiteId: $test_suite_id
                parameters: $parameters
                qaSetId: $qa_set_id
                runName: $run_name
              ) {
                runId
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "test_suite_id": test_suite_id,
            "parameters": parameters,
            "qa_set_id": qa_set_id,
            "run_name": run_name,
        }
        response = await self.execute(
            query=query, operation_name="startRun", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return StartRun.model_validate(data)

    async def run_param_info(self, **kwargs: Any) -> RunParamInfo:
        query = gql(
            """
            query RunParamInfo {
              runParameterInfo
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(
            query=query, operation_name="RunParamInfo", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return RunParamInfo.model_validate(data)

    async def run_status(self, run_id: str, **kwargs: Any) -> RunStatus:
        query = gql(
            """
            query RunStatus($runId: String!) {
              run(runId: $runId) {
                status
              }
            }
            """
        )
        variables: Dict[str, object] = {"runId": run_id}
        response = await self.execute(
            query=query, operation_name="RunStatus", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return RunStatus.model_validate(data)

    async def pull_run(self, run_id: str, **kwargs: Any) -> PullRun:
        query = gql(
            """
            query PullRun($runId: String!) {
              run(runId: $runId) {
                runId
                passPercentage
                status
                runId
                textSummary
                timestamp
                completedAt
                archived
                parameters
                testSuite {
                  title
                }
              }
              testResults(runId: $runId) {
                id
                llmOutput
                passPercentage
                passPercentageWithOptional
                resultJson
                humanEval
                humanFeedback
                test {
                  testId
                  inputUnderTest
                }
                metadata
              }
            }
            """
        )
        variables: Dict[str, object] = {"runId": run_id}
        response = await self.execute(
            query=query, operation_name="PullRun", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return PullRun.model_validate(data)

    async def list_runs(
        self,
        archived: Union[Optional[bool], UnsetType] = UNSET,
        suite_id: Union[Optional[str], UnsetType] = UNSET,
        **kwargs: Any
    ) -> ListRuns:
        query = gql(
            """
            query ListRuns($archived: Boolean, $suiteId: String) {
              runs(archived: $archived, suiteId: $suiteId) {
                runId
                passPercentage
                status
                runId
                textSummary
                timestamp
                completedAt
                archived
                parameters
                testSuite {
                  title
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"archived": archived, "suiteId": suite_id}
        response = await self.execute(
            query=query, operation_name="ListRuns", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return ListRuns.model_validate(data)

    async def create_or_update_test_suite(
        self, test_suite_id: str, title: str, description: str, **kwargs: Any
    ) -> CreateOrUpdateTestSuite:
        query = gql(
            """
            mutation createOrUpdateTestSuite($testSuiteId: String!, $title: String!, $description: String!) {
              updateTestSuite(
                testSuiteId: $testSuiteId
                title: $title
                description: $description
              ) {
                testSuite {
                  description
                  id
                  org
                  title
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "testSuiteId": test_suite_id,
            "title": title,
            "description": description,
        }
        response = await self.execute(
            query=query,
            operation_name="createOrUpdateTestSuite",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return CreateOrUpdateTestSuite.model_validate(data)

    async def add_batch_tests(
        self, tests: List[TestMutationInfo], create_only: bool, **kwargs: Any
    ) -> AddBatchTests:
        query = gql(
            """
            mutation addBatchTests($tests: [TestMutationInfo!]!, $createOnly: Boolean!) {
              batchUpdateTest(tests: $tests, createOnly: $createOnly) {
                tests {
                  checks
                  testId
                  crossVersionId
                  fileIds
                  inputUnderTest
                  tags
                  context
                  goldenOutput
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"tests": tests, "createOnly": create_only}
        response = await self.execute(
            query=query, operation_name="addBatchTests", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return AddBatchTests.model_validate(data)

    async def delete_test_suite(
        self, test_suite_id: str, **kwargs: Any
    ) -> DeleteTestSuite:
        query = gql(
            """
            mutation deleteTestSuite($testSuiteId: String!) {
              deleteSuite(suiteId: $testSuiteId) {
                success
              }
            }
            """
        )
        variables: Dict[str, object] = {"testSuiteId": test_suite_id}
        response = await self.execute(
            query=query, operation_name="deleteTestSuite", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return DeleteTestSuite.model_validate(data)

    async def update_global_checks(
        self, test_suite_id: str, checks: str, **kwargs: Any
    ) -> UpdateGlobalChecks:
        query = gql(
            """
            mutation updateGlobalChecks($testSuiteId: String!, $checks: String!) {
              updateGlobalChecks(testSuiteId: $testSuiteId, checks: $checks) {
                success
              }
            }
            """
        )
        variables: Dict[str, object] = {"testSuiteId": test_suite_id, "checks": checks}
        response = await self.execute(
            query=query,
            operation_name="updateGlobalChecks",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return UpdateGlobalChecks.model_validate(data)

    async def remove_old_tests(
        self, test_suite_id: str, in_use_tests: List[str], **kwargs: Any
    ) -> RemoveOldTests:
        query = gql(
            """
            mutation removeOldTests($testSuiteId: String!, $inUseTests: [String!]!) {
              removeUnusedTests(testSuiteId: $testSuiteId, inUseTests: $inUseTests) {
                success
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "testSuiteId": test_suite_id,
            "inUseTests": in_use_tests,
        }
        response = await self.execute(
            query=query, operation_name="removeOldTests", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return RemoveOldTests.model_validate(data)

    async def get_test_suite_data(
        self, suite_id: str, **kwargs: Any
    ) -> GetTestSuiteData:
        query = gql(
            """
            query getTestSuiteData($suiteId: String!) {
              testSuites(testSuiteId: $suiteId) {
                description
                id
                org
                title
                created
                globalChecks
              }
            }
            """
        )
        variables: Dict[str, object] = {"suiteId": suite_id}
        response = await self.execute(
            query=query,
            operation_name="getTestSuiteData",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return GetTestSuiteData.model_validate(data)

    async def get_test_data(self, suite_id: str, **kwargs: Any) -> GetTestData:
        query = gql(
            """
            query getTestData($suiteId: String!) {
              tests(testSuiteId: $suiteId) {
                checks
                testId
                crossVersionId
                fileIds
                inputUnderTest
                tags
                context
                goldenOutput
              }
            }
            """
        )
        variables: Dict[str, object] = {"suiteId": suite_id}
        response = await self.execute(
            query=query, operation_name="getTestData", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return GetTestData.model_validate(data)

    async def get_test_suites_with_count(self, **kwargs: Any) -> GetTestSuitesWithCount:
        query = gql(
            """
            query getTestSuitesWithCount {
              testSuitesWithCount {
                testSuites {
                  id
                  title
                  description
                  created
                  creator
                  lastModifiedBy
                  lastModifiedAt
                  folder {
                    id
                    name
                  }
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(
            query=query,
            operation_name="getTestSuitesWithCount",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return GetTestSuitesWithCount.model_validate(data)

    async def get_operators(self, **kwargs: Any) -> GetOperators:
        query = gql(
            """
            query getOperators {
              operators {
                nameInDoc
                isUnary
              }
            }
            """
        )
        variables: Dict[str, object] = {}
        response = await self.execute(
            query=query, operation_name="getOperators", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return GetOperators.model_validate(data)



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class TestResults(BaseModel):
    test_results: List[Optional["TestResultsTestResults"]] = Field(alias="testResults")


class TestResultsTestResults(BaseModel):
    id: str
    llm_output: str = Field(alias="llmOutput")
    pass_percentage: float = Field(alias="passPercentage")
    pass_percentage_with_optional: float = Field(alias="passPercentageWithOptional")
    result_json: Any = Field(alias="resultJson")
    human_eval: Optional[float] = Field(alias="humanEval")
    human_feedback: str = Field(alias="humanFeedback")
    test: "TestResultsTestResultsTest"
    metadata: Optional[Any]


class TestResultsTestResultsTest(BaseModel):
    test_id: str = Field(alias="testId")
    input_under_test: str = Field(alias="inputUnderTest")


TestResults.model_rebuild()
TestResultsTestResults.model_rebuild()



import json
import sys
from io import TextIOWrapper
from typing import Any, Dict

import click
from vals.cli.util import display_error_and_exit, prompt_user_for_suite
from vals.sdk.exceptions import ValsException
from vals.sdk.suite import create_suite, list_test_suites, pull_suite, update_suite
from vals.sdk.util import fe_host


@click.group(name="suite")
def suite_group():
    """
    Start, create, or view tests and test suites
    """
    pass


def parse_suite_interactive():
    title = click.prompt("Test Suite Title")
    while title == "":
        title = click.prompt("Title cannot be empty. Reenter")

    description = click.prompt("Test Suite Description")

    i = 1
    keep_generating_prompts = True
    tests = []
    while keep_generating_prompts:
        click.secho(f"---Test {i}---", bold=True)
        input_under_test = click.prompt("Input under test (e.g. the prompt)")

        keep_generating_criteria = True
        j = 1
        checks = []
        while keep_generating_criteria:
            operator = click.prompt(f"Operator {j}")
            criteria = click.prompt(f"Criteria {j}")
            checks.append({"criteria": criteria, "operator": operator})
            j += 1

            keep_generating_criteria = click.confirm("Keep Generating Checks?")

        i += 1

        tests.append({"input_under_test": input_under_test, "checks": checks})
        keep_generating_prompts = click.confirm("Keep generating tests?")

    return {"title": title, "description": description, "tests": tests}


def parse_suite_file(file):
    # TODO: Validate file format
    try:
        return json.load(file)
    except Exception as e:
        raise ValsException("The input file provided is not valid JSON")


def parse_suite(interactive: bool, file: TextIOWrapper) -> Dict[str, Any]:
    if not interactive and file is None:
        click.echo(
            "Either --interactive must be passed, or an input file should be specified"
        )
        sys.exit(1)

    if interactive:
        data = parse_suite_interactive()
    else:
        data = parse_suite_file(file)

    return data


@click.command(name="create")
@click.option(
    "--interactive",
    "-i",
    is_flag=True,
    help="Enable interactive mode instead of reading from file",
)
@click.argument("file", type=click.File("r"), required=False)
def create_command(interactive: bool, file: str):
    """
    Creates a new test suite.

    There are two modes. In normal operation, inputs are read from a JSON file:

    \tprl suite create <filename>

    In interactive mode, the user is prompted for values:

    \tprl suite create --interactive

        Requires authentication to use.
    """
    # try:
    data = parse_suite(interactive, file)

    try:
        suite_id = create_suite(data)
    except ValsException as e:
        display_error_and_exit(f"Could not create suite. {e.message}")

    # Execute the query on the transport
    click.secho("Successfully created test suite.", fg="green")
    click.secho(f"{fe_host()}/view?test_suite_id={suite_id}", bold=True)


@click.command(name="update")
@click.option(
    "--interactive",
    "-i",
    is_flag=True,
    help="Enable interactive mode instead of reading from file",
)
@click.argument("file", type=click.File("r"), required=False)
def update_command(interactive: bool, file: str):
    """
    Update the test and checks of an already existing suite
    """
    suite_id = prompt_user_for_suite()
    try:
        data = parse_suite(interactive, file)
        update_suite(suite_id, data)

        click.secho("Successfully updated test suite.", fg="green")
        click.secho(f"{fe_host()}/view?test_suite_id={suite_id}", bold=True)
    except ValsException as e:
        click.secho(e.message, fg="red")
    except Exception as e:
        click.secho("Suite Update Failed. Error:" + str(e), fg="red")


@click.command(name="list")
def list_command():
    """
    List test suites associated with this organization
    """
    suites = list_test_suites()

    suite_text = "\n".join([f"{i}: {s['title']}" for i, s in enumerate(suites)])
    click.echo(suite_text)


@click.command(name="pull")
@click.argument("file", type=click.File("w"), required=True)
def pull_command(file: TextIOWrapper):
    """
    Read a suite from the PRL server to a local JSON file.
    """
    suite_id = prompt_user_for_suite()
    try:
        output = pull_suite(suite_id)
        file.write(json.dumps(output, indent=2))

    except ValsException as e:
        display_error_and_exit(e.message)

    click.secho("Successfully pulled test suite.", fg="green")


suite_group.add_command(create_command)
suite_group.add_command(list_command)
suite_group.add_command(update_command)
suite_group.add_command(pull_command)



import json
from functools import wraps
from io import BytesIO
from time import time
from typing import Callable

import click
import pypandoc
import requests
from openai import OpenAI
from pypdf import PdfReader
from tqdm import tqdm
from vals.cli.suite import pull_suite, update_suite
from vals.sdk.auth import _get_auth_token
from vals.sdk.run import (
    Metadata,
    QuestionAnswerPair,
    _create_question_answer_set,
    get_run_url,
    start_run,
)
from vals.sdk.util import be_host
from vals.sdk.v2.util import parse_file_id

in_tokens = 0
out_tokens = 0


def _parse_test_suite_id_from_url(test_suite_url: str) -> str:
    start_index = test_suite_url.find("test_suite_id=") + len("test_suite_id=")
    return test_suite_url[start_index:]


def _read_file(file_id: str):
    response = requests.post(
        url=f"{be_host()}/download_file/?file_id={file_id}",
        headers={"Authorization": _get_auth_token()},
    )
    return BytesIO(response.content)


def read_pdf(file: BytesIO):
    """
    Convenience method to parse PDFs to strings
    """
    text = ""
    pdf_reader = PdfReader(file)
    num_pages = len(pdf_reader.pages)
    for page_number in range(num_pages):
        page = pdf_reader.pages[page_number]
        text += page.extract_text()
    return text


def read_docx(file: BytesIO):
    """
    Convenience method to parse docx files to strings
    """
    output = pypandoc.convert_text(file.read(), to="plain", format="docx")
    return output


def read_text(file: BytesIO):
    return file.read().decode("utf-8")


def run_evaluations(
    test_suite_url: str,
    generate_fn: Callable[[str], str],
    description="Ran automatically using the PRL SDK",
    maximum_threads=4,
    verbosity=1,
    model_under_test="sdk",
    **kwargs,
):
    """
    This function is meant to be used to test arbitrary models and pipelines with the Vals platform.

    It takes in a `generate_fn` and the url of a test suite.

    For every test input in the test suite, it queries generate_fn(test_input) to produce the
    output from the LLM model or application. It then updates the test suite with the results,
    and kicks off a new run.

    It returns the run_id of the run.

    See examples/sdk_example.py for example usage.
    """
    test_suite_id = _parse_test_suite_id_from_url(test_suite_url)

    suite_data = pull_suite(test_suite_id, include_id=True)

    if verbosity == 0:
        iterator = suite_data["tests"]
    else:
        iterator = tqdm(suite_data["tests"])

    global in_tokens, out_tokens
    metadata = {}

    uses_files = False
    uses_file_uids = False
    uses_context = False

    for test in suite_data["tests"]:
        if (
            "file_ids" in test
            and test["file_ids"] is not None
            and len(test["file_ids"])
        ):
            uses_files = True

        if (
            "file_uids" in test
            and test["file_uids"] is not None
            and len(test["file_uids"]) > 0
        ):
            uses_file_uids = True

        if (
            "context" in test
            and test["context"] is not None
            and len(test["context"]) > 0
        ):
            uses_context = True

    # Kwargs that we shouldn't register as extra parameters
    non_param_kwargs = {}

    qa_pairs = []
    for test in iterator:
        file_ids = []
        context = {}
        if uses_files:
            if "file_ids" in test and test["file_ids"] is not None:
                file_ids = test["file_ids"]

                files = {}
                for file_id in file_ids:
                    _, file_name, _ = parse_file_id(file_id)
                    files[file_name] = _read_file(file_id)

            non_param_kwargs["files"] = files

        if uses_file_uids:
            if "file_uids" in test and test["file_uids"] is not None:
                file_uids = test["file_uids"]
                non_param_kwargs["file_uids"] = file_uids
            else:
                non_param_kwargs["file_uids"] = []

        if uses_context:
            if "context" in test and test["context"] is not None:
                context = test["context"]
                non_param_kwargs["context"] = context
            else:
                non_param_kwargs["context"] = {}

        non_param_kwargs.update(kwargs)
        start = time()
        llm_output = generate_fn(test["input_under_test"], **non_param_kwargs)
        end = time()
        metadata = Metadata(
            in_tokens=in_tokens,
            out_tokens=out_tokens,
            duration_seconds=end - start,
        )
        in_tokens = 0
        out_tokens = 0

        qa_pairs.append(
            QuestionAnswerPair(
                input_under_test=test["input_under_test"],
                file_ids=file_ids,
                context=context,
                llm_output=llm_output,
                metadata=metadata,
                # TODO: This may be one version behind.
                test_id=test["id"],
            )
        )

    qa_set_id = _create_question_answer_set(
        test_suite_id,
        qa_pairs,
        model_id=model_under_test,
        parameters={
            "description": description,
            "model_under_test": model_under_test,
            **kwargs,
        },
    )

    run_id = start_run(
        test_suite_id,
        {
            "description": description,
            "maximum_threads": maximum_threads,
            "model_under_test": model_under_test,
            **kwargs,
        },
        qa_set_id=qa_set_id,
    )
    run_url = get_run_url(run_id)

    if verbosity >= 1:
        click.secho(
            "Successfully updated test suite with new fixed outputs and started a new run.",
            fg="green",
        )
        click.secho(run_url, bold=True)

    return run_id


def _wrap_chatcompletion(func: Callable):
    @wraps(func)
    def wrapper(**kwargs):
        response = func(**kwargs)
        global in_tokens, out_tokens
        in_tokens += response.usage.prompt_tokens
        out_tokens += response.usage.completion_tokens

        return response

    return wrapper


# External Facing
def patch(client: OpenAI):
    """
    Calling this function allows the Vals SDK to collect token metadata from any calls to OpenAI
    or models that use the OpenAI API.
    """
    client.chat.completions.create = _wrap_chatcompletion(
        client.chat.completions.create
    )
    return client



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, Optional

from pydantic import Field

from .base_model import BaseModel


class RunSummary(BaseModel):
    run: "RunSummaryRun"


class RunSummaryRun(BaseModel):
    run_id: str = Field(alias="runId")
    pass_percentage: Optional[float] = Field(alias="passPercentage")
    status: str
    run_id: str = Field(alias="runId")
    text_summary: str = Field(alias="textSummary")
    timestamp: Any
    archived: bool
    parameters: Any
    test_suite: "RunSummaryRunTestSuite" = Field(alias="testSuite")


class RunSummaryRunTestSuite(BaseModel):
    title: str


RunSummary.model_rebuild()
RunSummaryRun.model_rebuild()



import json
import os
from typing import Any, Dict, List

import requests
from gql import gql
from jsonschema import ValidationError, validate
from vals.sdk.auth import _get_auth_token
from vals.sdk.exceptions import ValsException
from vals.sdk.util import SUITE_SCHEMA_PATH, be_host, get_client

OPERATORS = [
    "includes",
    "logical_includes",
    "excludes",
    "equals",
    "mostly_equals",
    "includes_exactly",
    "includes_nearly_exactly",
    "excludes_exactly",
    "equals_exactly",
    "less_than_length",
    "answers",
    "not_answers",
    "is_concise",
    "is_coherent",
    "is_safe",
    "is_not_hallucinating",
    "is_not_generated",
    "no_legal_advice",
    "is_not_templated",
    "satisfies_statement",
    "consistent_phrasing",
    "valid_json",
    "valid_yaml",
    "regex",
    "matches_json_schema",
    "includes_any",
    "includes_any_v2",
    "list_format",
    "paragraph_format",
    "affirmative_answer",
    "negative_answer",
    "matches_tone",
    "check_salesiness",
    "matches_format",
    "grammar",
    "equal_intent",
    "capitalized_correctly",
    "is_polite",
    "progresses_conversation",
    "legacy_equals",
    "consistent_with_json",
    "consistent_with_context",
    "consistent_with_docs",
    "includes_only",
]

UNARY_OPERATORS = [
    "answers",
    "not_answers",
    "is_concise",
    "is_coherent",
    "is_safe",
    "is_not_hallucinating",
    "is_not_generated",
    "no_legal_advice",
    "is_not_templated",
    "valid_json",
    "valid_yaml",
    "list_format",
    "paragraph_format",
    "affirmative_answer",
    "negative_answer",
    "grammar",
    "capitalized_correctly",
    "is_polite",
    "progresses_conversation",
    "consistent_with_json",
    "consistent_with_context",
    "consistent_with_docs",
    "safety_consistency",
]


def create_suite(suite_data: Dict[str, Any]) -> str:
    """
    Method to create a test suite. Suite_data
    should be a JSON-like python dict that contains the information outlined in our docs.

    Returns the ID of the newly created suite.
    """
    _validate_suite(suite_data)
    suite_id = _create_test_suite(suite_data)

    files = _upload_files(suite_id, suite_data)

    _add_global_checks(suite_data, suite_id)
    _add_tests(suite_data, files, suite_id, create_only=True)

    return suite_id


def update_suite(suite_id: str, suite_data: Dict[str, Any]) -> None:
    """
    Method to update a test suite. data is in the same format as create_suite.
    """
    _validate_suite(suite_data)

    files = _upload_files(suite_id, suite_data)

    _add_tests(suite_data, files, suite_id)
    _add_global_checks(suite_data, suite_id)


def list_test_suites() -> List[Dict[str, Any]]:
    """

    Method to produce a list of test suites for a given org.

    """
    query = gql(
        f"""
        query getTestSuites {{
            testSuites {{
            description
            id
            org
            title
            created
            creator
            }}
            }}
        """
    )
    response = get_client().execute(query)

    # TODO: Error check
    return response["testSuites"]


def pull_suite(suite_id: str, include_id=False):
    """
    Get the JSON representation of a given test suite.
    """
    output = {}
    query = gql(
        f"""
        query getTestSuiteData {{
            testSuites(testSuiteId: "{suite_id}") {{
                description
                id
                org
                title
                created
                globalChecks
            }}
        }}
    """
    )

    response = get_client().execute(query)

    if len(response["testSuites"]) == 0:
        raise Exception(f"Unable to find test suite with id: {suite_id}")

    suite = response["testSuites"][0]
    output["title"] = suite["title"]
    output["description"] = suite["description"]

    query = gql(
        f"""
        query getTestData {{
            tests(testSuiteId: "{suite_id}") {{
                checks
                testId
                crossVersionId
                fileIds
                inputUnderTest
                sampleOutput
                goldenOutput
                sampleOutputType
                fileUids
                tags
                context
            }}
        }}    
        """
    )
    response = get_client().execute(query)
    raw_tests = response["tests"]

    tests = []
    for raw_test in raw_tests:
        test = {}
        if include_id:
            test["id"] = raw_test["testId"]
            test["cross_version_id"] = raw_test["crossVersionId"]

        test["input_under_test"] = raw_test["inputUnderTest"]

        if raw_test["fileIds"] is not None:
            file_ids = json.loads(raw_test["fileIds"])
            if len(file_ids) != 0:
                test["file_ids"] = file_ids

        if raw_test["fileUids"] is not None:
            file_uids = json.loads(raw_test["fileUids"])
            if len(file_uids) != 0:
                test["file_uids"] = file_uids

        if raw_test["sampleOutput"] != "":
            if raw_test["sampleOutputType"] == "file":
                test["file_fixed_output"] = raw_test["sampleOutput"]
            else:
                test["fixed_output"] = raw_test["sampleOutput"]

        if raw_test["goldenOutput"] != "":
            test["golden_output"] = raw_test["goldenOutput"]

        test["checks"] = json.loads(raw_test["checks"])
        context = json.loads(raw_test["context"])
        if len(context) != 0:
            test["context"] = context

        tags = json.loads(raw_test["tags"])

        if len(tags) != 0:
            test["tags"] = tags

        tests.append(test)

    output["tests"] = tests
    return output


def _validate_checks(checks):
    for check in checks:
        operator = check["operator"]
        if operator not in OPERATORS:
            raise ValsException(f"Unrecognized operator: {operator}")
        if operator not in UNARY_OPERATORS and "criteria" not in check:
            raise ValsException(
                f"'criteria' field must be specified for check with operator: '{operator}'"
            )


def _validate_suite(parsed_json):
    if not os.path.exists(SUITE_SCHEMA_PATH):
        raise ValsException(
            "Could not find schema file. The CLI tool is likely misconfigured."
        )

    # Use jsonschema to do most of our validation
    try:
        with open(SUITE_SCHEMA_PATH, "r") as schema_file:
            schema = json.load(schema_file)
            validate(instance=parsed_json, schema=schema)
    except ValidationError as e:
        raise ValsException(
            f"The file provided did not conform to the correct format. Validation Error: {e.message}. Look at the examples or the jsonschema to see the correct format."
        )

    if "global_checks" in parsed_json:
        _validate_checks(parsed_json["global_checks"])

    # We need to do some custom validation that JSON schema doesn't support
    tests = parsed_json["tests"]
    for test in tests:
        if "input_under_test" not in test and "file_under_test" not in test:
            raise ValsException(
                "For all tests, either 'input_under_test' or 'file_under_test' must be provided"
            )

        if "file_under_test" in test:
            fp = test["file_under_test"]
            if not os.path.exists(fp):
                raise ValsException(f"File does not exist: {fp}")
            if not os.path.isfile(fp):
                raise ValsException(f"Path is a directory: {fp}")

        if "file_uids" in test:
            if not isinstance(test["file_uids"], list):
                raise ValsException("file_uids must be a list")

        _validate_checks(test["checks"])


def _upload_file(suite_id: str, file_path: str) -> str:
    with open(file_path, "rb") as f:
        response = requests.post(
            f"{be_host()}/upload_file/?test_suite_id={suite_id}",
            files={"file": f},
            headers={"Authorization": _get_auth_token()},
        )
        if response.status_code != 200:
            raise Exception(f"Failed to upload file {file_path}")
        return response.json()["file_id"]


def _upload_files(suite_id: str, data: Dict[str, Any]):
    # Map from file path to file id
    files = {}
    for test in data["tests"]:
        if "file_under_test" in test:
            file_path = test["file_under_test"]
            if file_path not in files:
                files[file_path] = _upload_file(suite_id, file_path)

        if "files_under_test" in test:
            # We can either have a single file path, or a list of file paths
            file_path = test["files_under_test"]
            if type(file_path) == str:
                file_paths = [file_path]
            elif type(file_path) == list:
                file_paths = file_path
            for file_path in file_paths:
                files[file_path] = _upload_file(suite_id, file_path)

        if "file_fixed_output" in test:
            file_path = test["file_fixed_output"]
            if file_path not in files:
                files[file_path] = _upload_file(suite_id, file_path)

    return files


def _create_test_suite(data: Dict[str, Any]) -> str:
    query = gql(
        f"""
    mutation createTestSuite {{
        updateTestSuite(
            description: {json.dumps(data['description'])},
            testSuiteId: "0",
            title: "{data['title']}"
        ) {{
            testSuite {{
            description
            id
            org
            title
            }}
        }}
    }}
    """
    )
    result = get_client().execute(query)
    suite_id = result["updateTestSuite"]["testSuite"]["id"]
    return suite_id


def _add_global_checks(data, suite_id):
    if "global_checks" not in data:
        return

    query = gql(
        f"""
    mutation mutateGlobalChecks {{
        updateGlobalChecks (
            testSuiteId: "{suite_id}",
            checks: {json.dumps(json.dumps(data["global_checks"]))}
        ) {{
            success
        }}
    }}
    """
    )
    result = get_client().execute(query)


def _add_batch_to_suite(batch: List[str], create_only: bool):
    """
    Helper method to add a list of tests to a given suite
    """
    #
    query_str = f"""
        mutation addBatchTests {{
            batchUpdateTest(
                tests: [
                    {",".join(batch)}
                ],
                createOnly: {str(create_only).lower()}
            ) {{
                tests {{
                    testId
                }}
            }}
        }}
        """
    query = gql(query_str)
    response = get_client().execute(query)
    tests = response["batchUpdateTest"]["tests"]
    return [t["testId"] for t in tests]


def _add_tests(data, files, suite_id, create_only=False):
    test_ids = []
    batch = []
    i = 0

    for test in data["tests"]:
        # TODO: Escape chars better

        input_under_test = test["input_under_test"]

        # TODO: avoid double json
        checks = json.dumps(json.dumps(test["checks"]))
        # TODO: Do this server side

        if "fixed_output" in test:
            fixed_output = test["fixed_output"]
            fixed_output_type = "raw"
        elif "file_fixed_output" in test:
            fixed_output = files[test["file_fixed_output"]]
            fixed_output_type = "file"
        else:
            fixed_output = ""
            fixed_output_type = "raw"

        file_ids = []

        if "files_under_test" in test:
            file_path = test["files_under_test"]
            if type(file_path) == str:
                file_paths = [file_path]
            elif type(file_path) == list:
                file_paths = file_path

            file_ids = [files[file_path] for file_path in file_paths]
        # Backwards compatability for old suites
        elif "file_under_test" in test:
            file_paths = [test["file_under_test"]]

            file_ids = [files[file_path] for file_path in file_paths]
        elif "file_ids" in test:
            file_ids = test["file_ids"]

        if "file_uids" in test:
            file_uids = test["file_uids"]

        else:
            file_uids = []

        if "context" in test:
            context = json.dumps(json.dumps(test["context"]))
        else:
            context = json.dumps(json.dumps({}))

        if "golden_output" in test:
            golden_output = test["golden_output"]
        else:
            golden_output = ""

        tags = []
        if "tags" in test:
            tags = test["tags"]

        test_id = 0 if "id" not in test else test["id"]

        # We collate the tests we want to add in batches
        # When we have 100 tests in a batch, we add it to the suite.
        batch.append(
            f"""
            {{
                  sampleOutput: {json.dumps(fixed_output)},
                  sampleOutputType: "{fixed_output_type}",
                  checks: {checks}, 
                  inputUnderTest: {json.dumps(input_under_test)}, 
                  testSuiteId: "{suite_id}",
                  fileIds: {json.dumps(file_ids)}
                  fileUids: {json.dumps(file_uids)}
                  context: {context}
                  goldenOutput: {json.dumps(golden_output)}
                  tags: {json.dumps(tags)}
                  testId: "{test_id}"
            }}
            """
        )
        i += 1
        if i % 100 == 0:
            new_ids = _add_batch_to_suite(batch, create_only=create_only)
            test_ids.extend(new_ids)
            batch = []

    if len(batch) != 0:
        new_ids = _add_batch_to_suite(batch, create_only=create_only)
        test_ids.extend(new_ids)

    test_id_list = ", ".join([f'"{test_id}"' for test_id in test_ids])
    query = gql(
        f"""
            mutation removeOldTests {{
              removeUnusedTests(
                  testSuiteId: "{suite_id}",
                  inUseTests: [{test_id_list}]
                ) {{
                    success
                }}
            }}
            """
    )
    response = get_client().execute(query)

    # TODO: Check for errors



import hashlib
from io import BytesIO

import requests
from vals.graphql_client.client import Client as AriadneClient
from vals.sdk.auth import _get_auth_token
from vals.sdk.util import be_host


def get_ariadne_client() -> AriadneClient:
    """
    Use the new codegen-based client
    """

    return AriadneClient(
        url=f"{be_host()}/graphql/",
        headers={"Authorization": _get_auth_token()},
    )


def md5_hash(file) -> str:
    """Produces an md5 hash of the file."""
    hasher = hashlib.md5()
    while chunk := file.read(8192):  # Read in 8 KB chunks
        hasher.update(chunk)
    hash = hasher.hexdigest()
    file.seek(0)
    return hash


def parse_file_id(file_id: str) -> tuple[str, str, str | None]:
    if len(file_id) >= 37 and file_id[-37] == ";":
        # This is a heuristic to check if we are in the old file id regime.
        # I checked and all previously uploaded files should match this
        # Counting the number of slashes ensures that it's not a new file id, that
        # happens to have a semicolon in the wrong spot
        org, rest = file_id.split("/")
        filename, test_suite_id = rest.split(";")
        return org, filename, None

    tokens = file_id.split("/")
    if len(tokens) != 2:
        raise Exception(f"Improperly formatted file_id: {file_id}")

    org, filename_with_hash = tokens

    hash, filename = filename_with_hash.split("-", 1)

    if len(hash) != 32:
        raise Exception(f"Improperly formatted file_id: {file_id}")

    return org, filename, hash


def read_file(file_id: str) -> BytesIO:
    response = requests.post(
        url=f"{be_host()}/download_file/?file_id={file_id}",
        headers={"Authorization": _get_auth_token()},
    )
    return BytesIO(response.content)



# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class GetTestSuites(BaseModel):
    test_suites: List["GetTestSuitesTestSuites"] = Field(alias="testSuites")


class GetTestSuitesTestSuites(BaseModel):
    description: str
    id: str
    org: str
    title: str
    created: Any
    creator: str


GetTestSuites.model_rebuild()



# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class ListRuns(BaseModel):
    runs: List["ListRunsRuns"]


class ListRunsRuns(BaseModel):
    run_id: str = Field(alias="runId")
    pass_percentage: Optional[float] = Field(alias="passPercentage")
    status: str
    run_id: str = Field(alias="runId")
    text_summary: str = Field(alias="textSummary")
    timestamp: datetime
    completed_at: Optional[datetime] = Field(alias="completedAt")
    archived: bool
    parameters: Any
    test_suite: "ListRunsRunsTestSuite" = Field(alias="testSuite")


class ListRunsRunsTestSuite(BaseModel):
    title: str


ListRuns.model_rebuild()
ListRunsRuns.model_rebuild()



import inspect
import json
import os
from time import time
from typing import Any, cast

from pydantic import BaseModel, PrivateAttr
from tqdm import tqdm
from vals.graphql_client.client import Client
from vals.graphql_client.input_types import MetadataType, QuestionAnswerPairInputType
from vals.sdk.run import _get_default_parameters
from vals.sdk.suite import _upload_file, _validate_suite
from vals.sdk.v2 import patch
from vals.sdk.v2.run import Run
from vals.sdk.v2.types import (
    Check,
    ModelFunctionType,
    ModelFunctionWithFilesAndContextType,
    SimpleModelFunctionType,
    Test,
    TestSuiteMetadata,
)
from vals.sdk.v2.util import get_ariadne_client, md5_hash, parse_file_id, read_file


class Suite(BaseModel):
    id: str | None = None

    title: str
    description: str = ""
    global_checks: list[Check] = []
    tests: list[Test] = []

    """
    Helper to see if we've instantiated the suite, but haven't 
    fetched it from the server yet. 
    """
    _client: Client = PrivateAttr(default_factory=get_ariadne_client)

    @classmethod
    async def list_suites(cls) -> list[TestSuiteMetadata]:
        """
        Generate a list of all the test suites on the server.
        """
        client = get_ariadne_client()
        gql_response = await client.get_test_suites_with_count()
        gql_suites = gql_response.test_suites_with_count.test_suites
        return [TestSuiteMetadata.from_graphql(gql_suite) for gql_suite in gql_suites]

    @classmethod
    async def from_id(cls, suite_id: str) -> "Suite":
        """
        Create a new local test suite based on the data from the server.
        """

        client = get_ariadne_client()
        suites_list = await client.get_test_suite_data(suite_id)
        if len(suites_list.test_suites) == 0:
            raise Exception("Couldn't find suite with id: " + suite_id)

        suite_data = suites_list.test_suites[0]
        title = suite_data.title
        description = suite_data.description

        global_checks = []
        if suite_data.global_checks is not None:
            global_checks = [
                Check.from_graphql(check)
                for check in json.loads(suite_data.global_checks)
            ]

        test_data = await client.get_test_data(suite_id)

        tests = []
        for graphql_test in test_data.tests:
            test = Test.from_graphql_test(graphql_test)
            tests.append(test)

        return cls(
            id=suite_id,
            title=title,
            description=description,
            global_checks=global_checks,
            tests=tests,
        )

    @classmethod
    async def from_dict(cls, data: dict[str, Any]) -> "Suite":
        """
        Imports the test suite from a dictionary - useful if
        importing from the old format.
        """
        _validate_suite(data)

        title = data["title"]
        description = data["description"]
        global_checks = [
            Check.from_graphql(check_dict)
            for check_dict in data.get("global_checks", [])
        ]
        tests = [
            Test(
                input_under_test=test["input_under_test"],
                checks=[
                    Check.from_graphql(check_dict) for check_dict in test["checks"]
                ],
                golden_output=test.get("golden_output", ""),
                tags=test.get("tags", []),
                files_under_test=test.get("files_under_test", []),
            )
            for test in data["tests"]
        ]
        return cls(
            title=title,
            description=description,
            global_checks=global_checks,
            tests=tests,
        )

    @classmethod
    async def from_json_file(cls, file_path: str) -> "Suite":
        """
        Imports the test suite from a local JSON file.
        """
        with open(file_path, "r") as f:
            data = json.load(f)
        return await cls.from_dict(data)

    async def create(self) -> None:
        """
        Creates the test suite on the server.
        """
        if self.id is not None:
            raise Exception("This suite has already been created.")

        await self._validate_tests()

        # Create suite object on the server
        suite = await self._client.create_or_update_test_suite(
            "0", self.title, self.description
        )
        if suite.update_test_suite is None:
            raise Exception("Unable to update the test suite.")
        self.id = suite.update_test_suite.test_suite.id

        await self._upload_global_checks()
        await self._upload_files()
        await self._upload_tests(create_only=True)

    async def delete(self) -> None:
        """
        Deletes the test suite from the server.
        """
        if self.id is None:
            raise Exception(
                "This suite has not been created yet, so there's nothing to delete"
            )

        await self._client.delete_test_suite(self.id)

    async def update(self) -> None:
        """
        Pushes any local changes to the the test suite object to the server.
        """
        if self.id is None:
            raise Exception(
                "This suite has not been created yet, so there's nothing to update"
            )

        await self._client.create_or_update_test_suite(
            self.id, self.title, self.description
        )

        await self._upload_files()
        await self._upload_global_checks()
        await self._upload_tests(create_only=False)

        # Remove any tests that are no longer used after the update.
        await self._client.remove_old_tests(self.id, [test.id for test in self.tests])

    async def run(
        self,
        model_under_test: str | None = None,
        model_function: ModelFunctionType | None = None,
        run_name: str | None = None,
        wait_for_completion: bool = False,
        # TODO: Type parameters properly.
        parameters: dict[str, int | float | str | bool] = {},
    ) -> Run:
        """
        Runs the test suite. This can be used in one of two ways.

        First, if "model_under_test" is provided, and "model_function" is not, then we will run that stock model. The supported models
        are the same ones in the "Model Under Test" dropdown in the Web App - e.g., "gpt-4o", "claude-3-5-sonnet", etc.

        Second, if "model_function" is provided, then we will use that function to generate responses from the LLM locally.
        For example, your model function may look like this:

        def my_model_function(input: str) -> str:
            # Replace with your custom logic / prompt chains, etc.
            output = OpenAi.chat.completions.create(
                model="gpt-4o",
                messages=[{"role": "user", "content": input}],
            )
            return output.choices[0].message.content

        When using the model_function mode, the model_under_test parameter is just used for bookkeeping purposes -
        it's what shows up as the model in the frontend.
        """
        if self.id is None:
            raise Exception(
                "This suite has not been created yet. Call suite.create() before calling suite.run()"
            )

        if model_under_test is None and model_function is None:
            raise Exception(
                "One of 'model_under_test' or 'model_function' must be provided."
            )

        # Use the default parameters, and then override with any user-provided parameters.
        _default_parameters = _get_default_parameters()
        parameters = {**_default_parameters, **parameters}
        if model_under_test is not None:
            parameters["model_under_test"] = model_under_test

        # Collate the local output pairs if we're using a model function.
        qa_set_id = None
        if model_function is not None:
            qa_set_id = await self._create_qa_set(
                model_function, parameters, model_under_test
            )

        # Start and pull the run.
        response = await self._client.start_run(
            self.id, parameters, qa_set_id, run_name
        )
        if response.start_run is None:
            raise Exception("Unable to start the run.")

        run_id = response.start_run.run_id
        run = await Run.from_id(run_id)
        if wait_for_completion:
            await run.wait_for_run_completion()
        await run.pull()

        return run

    async def _upload_files(self):
        """
        Helper method to upload the files to the server.
        Uploads any tests.files_under_test that haven't been uploaded yet.
        """
        if self.id is None:
            raise Exception("This suite has not been created yet.")

        file_name_and_hash_to_file_id = {}

        # First, we populate the dictionary of files that have already been uploaded
        # for this test suite.
        for test in self.tests:
            for file_id in test._file_ids:
                _, name, _hash = parse_file_id(file_id)
                file_name_and_hash_to_file_id[(name, _hash)] = file_id

        # Next, for files we haven't uploaded yet, we upload them
        # and add them to the dictionary
        for test in self.tests:
            if len(test.files_under_test) != 0:
                file_ids = []
                for file_path in test.files_under_test:
                    if not os.path.exists(file_path):
                        raise Exception(f"File does not exist: {file_path}")

                    # First, we compute the filename and the hash of its contents
                    with open(file_path, "rb") as f:
                        file_hash = md5_hash(f)
                    filename = os.path.basename(file_path)
                    name_hash_tuple = (filename, file_hash)

                    # If the file hasn't been uploaded yet, we upload it
                    if name_hash_tuple not in file_name_and_hash_to_file_id:
                        file_id = _upload_file(self.id, file_path)
                        file_name_and_hash_to_file_id[name_hash_tuple] = file_id

                    # Either way, we add the file id to the test.
                    file_ids.append(file_name_and_hash_to_file_id[name_hash_tuple])

                # Main downside of this approach is that we can only overwrite files
                # we can't add or remove just one file.
                test._file_ids = file_ids

    async def _upload_tests(self, create_only: bool = True) -> None:
        """
        Helper method to upload the tests to the server in batches of 100.
        """
        if self.id is None:
            raise Exception("This suite has not been created yet.")

        # Upload tests in batches of 100
        created_tests = []
        test_mutations = [test.to_test_mutation_info(self.id) for test in self.tests]
        for i in range(0, len(test_mutations), 100):
            batch = test_mutations[i : i + 100]
            batch_result = await self._client.add_batch_tests(
                tests=batch,
                create_only=create_only,
            )
            if batch_result.batch_update_test is None:
                raise Exception("Unable to add tests to the test suite.")
            created_tests.extend(batch_result.batch_update_test.tests)

        # Update the local suite with the new tests to ensure everything is in sync.
        self.tests = [Test.from_graphql_test(test) for test in created_tests]

    async def _validate_tests(self) -> None:
        """Helper method to ensure that all operator strings are correct."""
        operators = (await self._client.get_operators()).operators
        operators_dict = {
            op.name_in_doc: op for op in operators
        }  # Easy lookup by name.

        for test in self.tests:
            for check in test.checks:
                if check.operator not in operators_dict:
                    raise ValueError(f"Invalid operator: {check.operator}")
                operator = operators_dict[check.operator]
                if operator.is_unary and (
                    check.criteria is None or check.criteria == ""
                ):
                    raise ValueError(
                        "Operator {operator} is unary, but no criteria was provided."
                    )

    async def _upload_global_checks(self) -> None:
        """
        Helper method to upload the global checks to the server.
        """
        if self.id is None:
            raise Exception("This suite has not been created yet.")

        await self._client.update_global_checks(
            self.id,
            json.dumps([gc.model_dump(exclude_none=True) for gc in self.global_checks]),
        )

    async def _create_qa_set(
        self,
        model_function: ModelFunctionType,
        parameters: dict[str, int | float | str | bool] = {},
        model_under_test: str | None = None,
    ) -> str | None:
        """
        Helper function to create a question-answer set from a model function.
        A question-answer set is just a set of inputs to the LLM, and the LLM's responses.
        """
        if self.id is None:
            raise Exception(
                "This suite has not been created yet, so we can't create a QA set from it."
            )

        # Pull latest suite data to ensure we're up to date
        updated_suite = await Suite.from_id(self.id)
        for field in self.__fields__:
            setattr(self, field, getattr(updated_suite, field))

        # Inspect the model function to determine if it takes 1 or 3 parameters
        # We dynamically change our call signature based on what the user passes in.
        sig = inspect.signature(model_function)
        num_params = len(sig.parameters)
        if num_params not in [1, 3]:
            raise ValueError("Model function must accept either 1 or 3 parameters")
        is_simple_model_function = num_params == 1

        # Query the LLM for each test.
        qa_pairs: list[QuestionAnswerPairInputType] = []
        for test in tqdm(self.tests):
            files = {}
            for file_id in test._file_ids:
                _, file_name, _ = parse_file_id(file_id)
                files[file_name] = read_file(file_id)

            time_start = time()
            in_tokens_start = patch.in_tokens
            out_tokens_start = patch.out_tokens

            if is_simple_model_function:
                casted_model_function = cast(SimpleModelFunctionType, model_function)
                llm_output = casted_model_function(test.input_under_test)
            else:
                casted_model_function = cast(
                    ModelFunctionWithFilesAndContextType, model_function
                )
                llm_output = casted_model_function(
                    test.input_under_test, files, test.context
                )

            time_end = time()
            in_tokens_end = patch.in_tokens
            out_tokens_end = patch.out_tokens

            qa_pairs.append(
                QuestionAnswerPairInputType(
                    input_under_test=test.input_under_test,
                    file_ids=test._file_ids,
                    context=test.context,
                    llm_output=llm_output,
                    metadata=MetadataType(
                        in_tokens=in_tokens_end - in_tokens_start,
                        out_tokens=out_tokens_end - out_tokens_start,
                        duration_seconds=time_end - time_start,
                    ),
                    test_id=test.id,
                )
            )

        response = await self._client.create_question_answer_set(
            self.id,
            qa_pairs,
            parameters,
            model_under_test or "sdk",
        )
        if response.create_question_answer_set is None:
            raise Exception("Unable to create the question-answer set.")

        return response.create_question_answer_set.question_answer_set.id



import asyncio
import json
from datetime import datetime
from typing import Any

import requests
from pydantic import BaseModel, PrivateAttr
from vals.graphql_client import Client
from vals.graphql_client.pull_run import PullRun
from vals.sdk.exceptions import ValsException
from vals.sdk.util import be_host, fe_host
from vals.sdk.v2.types import RunStatus, TestResult
from vals.sdk.v2.util import _get_auth_token, get_ariadne_client


class Run(BaseModel):
    id: str
    test_suite_id: str
    test_suite_title: str
    pass_percentage: float | None
    status: str
    archived: bool
    text_summary: str
    timestamp: datetime
    completed_at: datetime | None
    parameters: dict[str, Any]
    test_results: list[TestResult]

    _client: Client = PrivateAttr(default_factory=get_ariadne_client)

    @staticmethod
    def _create_from_pull_result(run_id: str, result: PullRun) -> "Run":
        """Helper method to create a Run instance from a pull_run query result"""
        return Run(
            id=run_id,
            test_results=[
                TestResult.from_graphql(test_result)
                for test_result in result.test_results
            ],
            pass_percentage=result.run.pass_percentage,
            text_summary=result.run.text_summary,
            archived=result.run.archived,
            parameters=json.loads(result.run.parameters),
            test_suite_title=result.run.test_suite.title,
            test_suite_id=result.run.test_suite.title,
            status=result.run.status,
            timestamp=result.run.timestamp,
            completed_at=result.run.completed_at,
        )

    @classmethod
    async def from_id(cls, run_id: str) -> "Run":
        """Pull most recent metadata and test results from the vals servers."""
        client = get_ariadne_client()
        result = await client.pull_run(run_id=run_id)
        return cls._create_from_pull_result(run_id, result)

    @property
    def url(self) -> str:
        return f"{fe_host()}/results?run_id={self.id}"

    async def pull(self) -> None:
        """Update this Run instance with latest data from vals servers."""
        result = await self._client.pull_run(run_id=self.id)
        updated = self._create_from_pull_result(self.id, result)
        # TODO: There's probably a better way to update the object.
        for field in updated.__fields__:
            setattr(self, field, getattr(updated, field))

    async def run_status(self) -> RunStatus:
        """Get the status of a run"""

        result = await self._client.run_status(run_id=self.id)
        self.status = result.run.status
        return RunStatus(result.run.status)

    async def wait_for_run_completion(
        self,
    ) -> RunStatus:
        """
        Block a process until a given run has finished running.

        Returns the status of the run after completion.
        """

        await asyncio.sleep(0.5)
        status = "in_progress"
        while status == "in_progress":
            status = await self.run_status()
            await asyncio.sleep(0.5)

        return RunStatus(status)

    async def to_csv(self, file_path: str) -> None:
        """Get the CSV results of a run, as bytes."""

        with open(file_path, "wb") as f:
            response = requests.post(
                url=f"{be_host()}/export_results_to_file/?run_id={self.id}",
                headers={"Authorization": _get_auth_token()},
            )

            if response.status_code != 200:
                raise ValsException(
                    "Received Error from Vals Servers: " + response.text
                )

            f.write(response.content)



"""
This example is similar to sdk_example.py, except we create our test suite in-line as well.
"""

import os

from openai import OpenAI
from vals.sdk.sdk import patch, run_evaluations
from vals.sdk.suite import create_suite

client = patch(OpenAI(api_key=os.environ.get("OPEN_AI_KEY")))

# This is the definition of our test suite, the same way
# one would define it in the platform
suite_data = {
    "title": "Example Suite [SDK]",
    "description": "An example suite created from the Vals SDK.",
    "tests": [
        {
            "input_under_test": "Who was the third president?",
            "checks": [
                {"operator": "grammar"},
                {"operator": "includes", "criteria": "Thomas Jefferson"},
            ],
        }
    ],
}

# This function creates a suite on the platform
suite_id = create_suite(suite_data=suite_data)


def test_function(test_input: str):
    prompt = "You are a pirate, answer in the speaking style of a pirate.\n\n"
    temp = 0.2

    gpt_client = OpenAI(api_key=os.environ.get("OPEN_AI_KEY"))
    response = gpt_client.chat.completions.create(
        model="gpt-4",
        messages=[{"role": "user", "content": prompt + test_input}],
        temperature=temp,
    )
    return response.choices[0].message.content


run_id = run_evaluations(
    f"https://platform.vals.ai/view?test_suite_id={suite_id}",
    test_function,
)
print(f"Run Id: {run_id}")



# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class PullRun(BaseModel):
    run: "PullRunRun"
    test_results: List["PullRunTestResults"] = Field(alias="testResults")


class PullRunRun(BaseModel):
    run_id: str = Field(alias="runId")
    pass_percentage: Optional[float] = Field(alias="passPercentage")
    status: str
    run_id: str = Field(alias="runId")
    text_summary: str = Field(alias="textSummary")
    timestamp: datetime
    completed_at: Optional[datetime] = Field(alias="completedAt")
    archived: bool
    parameters: Any
    test_suite: "PullRunRunTestSuite" = Field(alias="testSuite")


class PullRunRunTestSuite(BaseModel):
    title: str


class PullRunTestResults(BaseModel):
    id: str
    llm_output: str = Field(alias="llmOutput")
    pass_percentage: float = Field(alias="passPercentage")
    pass_percentage_with_optional: float = Field(alias="passPercentageWithOptional")
    result_json: Any = Field(alias="resultJson")
    human_eval: Optional[float] = Field(alias="humanEval")
    human_feedback: str = Field(alias="humanFeedback")
    test: "PullRunTestResultsTest"
    metadata: Optional[Any]


class PullRunTestResultsTest(BaseModel):
    test_id: str = Field(alias="testId")
    input_under_test: str = Field(alias="inputUnderTest")


PullRun.model_rebuild()
PullRunRun.model_rebuild()
PullRunTestResults.model_rebuild()



"""
This example is similar to sdk_example.py, except we create our test suite in-line as well.

We also add an additional file, and use consistent_with_docs to check it. 

NOTE: You should run this from the `examples/` folder. 
"""

import os
from io import BytesIO

from openai import OpenAI
from vals.sdk.sdk import patch, read_docx, run_evaluations
from vals.sdk.suite import create_suite

client = patch(OpenAI(api_key=os.environ.get("OPEN_AI_KEY")))

# This is the definition of our test suite, the same way
# one would define it in the platform
suite_data = {
    "title": "Example Suite - Files [SDK]",
    "description": "An example suite created from the Vals SDK.",
    "tests": [
        {
            "input_under_test": "Does this SAFE have an MFN clause?",
            "file_under_test": "data_files/postmoney_safe.docx",
            "checks": [
                {"operator": "grammar"},
                {"operator": "negative_answer"},
                {"operator": "consistent_with_docs"},
            ],
        }
    ],
}

# This function creates a suite on the platform
suite_id = create_suite(suite_data=suite_data)


def test_function(test_input: str, files: dict[str, BytesIO]):

    prompt = f"You are a pirate, answer in the speaking style of a pirate.\n\n Use the following documents to inform your answer:\n\n\n"

    for filename, file in files.items():
        # We could equivalently use read_pdf here, based on the extension.
        prompt += filename + ":\n"
        prompt += read_docx(file) + "\n\n"

    temp = 0.1

    gpt_client = OpenAI(api_key=os.environ.get("OPEN_AI_KEY"))
    response = gpt_client.chat.completions.create(
        model="gpt-4o",
        messages=[{"role": "user", "content": prompt + test_input}],
        temperature=temp,
    )
    return response.choices[0].message.content


run_id = run_evaluations(
    f"https://dev.platform.vals.ai/view?test_suite_id={suite_id}",
    test_function,
)
print(f"Run Id: {run_id}")



# Generated by ariadne-codegen
# Source: http://localhost:8000/graphql/

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import RunResultSortField, RunStatus, SortOrder, TestSuiteSortField


class FilterOptionsInput(BaseModel):
    limit: Optional[int] = None
    offset: Optional[int] = None
    search: Optional[str] = None
    folder_id: Optional[str] = Field(alias="folderId", default=None)
    test_suite_id: Optional[str] = Field(alias="testSuiteId", default=None)
    sort_by: Optional[TestSuiteSortField] = Field(alias="sortBy", default=None)
    sort_order: Optional[SortOrder] = Field(alias="sortOrder", default=None)


class RunResultFilterOptionsInput(BaseModel):
    limit: Optional[int] = None
    offset: Optional[int] = None
    search: Optional[str] = None
    status: Optional[List[Optional[RunStatus]]] = None
    archived: Optional[bool] = None
    run_by: Optional[List[str]] = Field(alias="runBy", default=None)
    models_under_test: Optional[List[str]] = Field(
        alias="modelsUnderTest", default=None
    )
    suite_id: Optional[str] = Field(alias="suiteId", default=None)
    sort_by: Optional[RunResultSortField] = Field(alias="sortBy", default=None)
    sort_order: Optional[SortOrder] = Field(alias="sortOrder", default=None)


class TestMutationInfo(BaseModel):
    test_suite_id: str = Field(alias="testSuiteId")
    test_id: str = Field(alias="testId")
    input_under_test: str = Field(alias="inputUnderTest")
    checks: str
    sample_output: Optional[str] = Field(alias="sampleOutput", default=None)
    sample_output_type: Optional[str] = Field(alias="sampleOutputType", default=None)
    golden_output: Optional[str] = Field(alias="goldenOutput", default=None)
    file_ids: Optional[List[Optional[str]]] = Field(alias="fileIds", default=None)
    context: Optional[str] = None
    tags: Optional[List[Optional[str]]] = None
    file_uids: Optional[List[Optional[str]]] = Field(alias="fileUids", default=None)


class PerCheckHumanReviewInputType(BaseModel):
    id: int
    binary_human_eval: Optional[int] = Field(alias="binaryHumanEval", default=None)
    is_flagged: Optional[bool] = Field(alias="isFlagged", default=None)


class CheckInputType(BaseModel):
    operator: str
    criteria: str
    modifiers: Any


class FixedOutputInputType(BaseModel):
    label: str
    text: str


class QuestionAnswerPairInputType(BaseModel):
    input_under_test: str = Field(alias="inputUnderTest")
    file_ids: Optional[List[Optional[str]]] = Field(alias="fileIds", default=None)
    context: Optional[Any] = None
    llm_output: str = Field(alias="llmOutput")
    metadata: Optional["MetadataType"] = None
    test_id: Optional[str] = Field(alias="testId", default=None)


class MetadataType(BaseModel):
    in_tokens: int = Field(alias="inTokens")
    out_tokens: int = Field(alias="outTokens")
    duration_seconds: float = Field(alias="durationSeconds")


QuestionAnswerPairInputType.model_rebuild()



import json
from datetime import datetime
from io import BytesIO

import click
from vals.sdk.run import (
    get_csv,
    get_run_results,
    get_run_url,
    run_status,
    run_summary,
    start_run,
)
from vals.sdk.util import fe_host

from ..sdk.exceptions import ValsException
from .util import display_error_and_exit, prompt_user_for_suite


@click.group(name="run")
def run_group():
    """
    Commands relating to starting or viewing runs
    """
    pass


@click.command(name="start")
@click.argument("param-file", type=click.File("r"), required=False, default=None)
@click.option("-s", "--suite-id", required=False)
def start_click_command(param_file, suite_id: str = None):
    """
    Start a new run of a test suite.
    """
    if param_file is not None:
        try:
            parameters = json.load(param_file)
        except Exception:
            display_error_and_exit("Config file was not valid JSON")
    else:
        parameters = {}

    if suite_id is None:
        suite_id = prompt_user_for_suite()

    try:
        run_id = start_run(suite_id, parameters)
    except ValsException as e:
        display_error_and_exit(e.message)

    click.secho("Successfully started run.", fg="green")
    click.secho(get_run_url(run_id), bold=True)


@click.command(name="get-csv")
@click.argument("run-id", type=click.STRING, required=True)
@click.argument("file", type=click.File("wb"), required=True)
def get_csv_click_command(run_id, file: BytesIO):
    """
    Get the CSV file with run results for a given run.

    This is equivalent to clicking 'Export to CSV' on the Run Results
    page within the website.

    Pass the path to a file where the CSV file should be downloaded.
    """

    try:
        csv_bytes = get_csv(run_id)
        file.write(csv_bytes)

    except ValsException as e:
        display_error_and_exit(e.message)

    click.secho("Successfully downloaded the result CSV.", fg="green")


@click.command(name="list-results")
@click.option(
    "--show-archived",
    is_flag=True,
    show_default=True,
    default=False,
    help="When enabled, archived runs are displayed in the output",
)
@click.option(
    "--ids-only",
    is_flag=True,
    show_default=True,
    default=False,
    help="When enabled, displays a list of only run ids with no additional information",
)
@click.option("--suite-id", required=False, help="Display")
def list_results_click_command(show_archived: bool, ids_only: bool, suite_id: str):
    """
    Display a list of run results.
    """
    run_results = get_run_results(show_archived, suite_id)
    if ids_only:
        for result in run_results:
            print(result["runId"])

    else:
        click.secho(
            f"{'Title':40} {'Timestamp':24} {'Status':13} {'Pass %':8} {'Archived':9}",
            bold=True,
        )
        for result in run_results:
            localtz = datetime.now().astimezone().tzinfo
            timestamp = datetime.strptime(
                result["timestamp"], "%Y-%m-%dT%H:%M:%S.%f%z"
            ).astimezone(localtz)
            # Print the timestamp in our own arbitrary format
            timestamp_str = datetime.strftime(timestamp, "%Y-%m-%d %H:%M %Z")
            pass_percentage = (
                "N/A"
                if result["passPercentage"] is None
                else f"{result['passPercentage'] * 100:.2f}"
            )
            click.secho(
                f"{result['testSuite']['title'][:38]:40} {timestamp_str:24} {result['status']:13} {pass_percentage:8} {'yes' if result['archived'] else 'no':10}",
            )


@click.command(name="status")
@click.argument("run-id", type=click.STRING, required=True)
def status_click_command(run_id):
    "CLI command to get the current status of a single run (error, in_progress, success)"
    status = run_status(run_id)
    click.echo(status)


@click.command(name="summary")
@click.argument("run-id", type=click.STRING, required=True)
def summary_click_command(run_id: str):
    """CLI command to get the current top-line result of a run"""
    dict_result = run_summary(run_id)
    click.echo(json.dumps(dict_result))


run_group.add_command(start_click_command)
run_group.add_command(list_results_click_command)
run_group.add_command(get_csv_click_command)
run_group.add_command(status_click_command)
run_group.add_command(summary_click_command)
